import os
import cv2
import numpy as np
from pkg_resources import Requirement, resource_filename


class FaceDnn(object):

    def __init__(self,
                 prototxt_file: str = None,
                 caffemodel_file: str = None,
                 threshold: float = 0.5,
                 scale_factor: float = 1.0,
                 size: tuple = (300, 300),
                 mean: tuple = (104.0, 177.0, 123.0)):
        if prototxt_file is None:
            prototxt = resource_filename(
                Requirement.parse('cv2utils'),
                'cv2utils' + os.path.sep + 'data' + os.path.sep + 'deploy.prototxt.txt')
        else:
            prototxt = prototxt_file

        if caffemodel_file is None:
            caffemodel = resource_filename(
                Requirement.parse('cv2utils'),
                'cv2utils' + os.path.sep + 'data' + os.path.sep + 'res10_300x300_ssd_iter_140000_fp16.caffemodel')
        else:
            caffemodel = caffemodel_file

        self.network = cv2.dnn.readNetFromCaffe(prototxt, caffemodel)
        self.threshold = threshold
        self.scale_factor = scale_factor
        self.size = size
        self.mean = mean

    def detect_faces(self, image):

        if image is None or not hasattr(image, "shape"):
            raise ValueError("Image not valid.")

        all_detections = []

        origin_h, origin_w = image.shape[:2]

        blob = cv2.dnn.blobFromImage(
            cv2.resize(image, self.size), self.scale_factor, self.size, self.mean)
        self.network.setInput(blob)

        detections = self.network.forward()

        for i in range(detections.shape[2]):

            confidence = detections[0, 0, i, 2]

            if confidence > self.threshold:

                bounding_box = detections[0, 0, i, 3:7] * np.array(
                    [origin_w, origin_h, origin_w, origin_h])
                x_start, y_start, x_end, y_end = bounding_box.astype('int')

                all_detections.append({
                    'label': 'face',
                    'confidence': float(confidence),
                    'box': [int(x_start), int(y_start), int(x_end), int(y_end)]
                })

        return all_detections



from cv2utils.haar_cascade import FaceCascade
from cv2utils.haar_cascade import EyeCascade
from cv2utils.opencv_dnn import FaceDnn

__all__ = ["FaceCascade",
           "EyeCascade",
           "FaceDnn"]



import os
import cv2
from pkg_resources import Requirement, resource_filename


class FaceCascade(object):

    def __init__(self,
                 weights_file: str = None,
                 scale_factor: float = 1.1,
                 min_neighbors: int = 5,
                 min_size: tuple = (10, 10)):

        if weights_file is None:
            weights_file = resource_filename(
                Requirement.parse('cv2utils'),
                'cv2utils' + os.path.sep + 'data' + os.path.sep + 'haarcascade_frontalface_default.xml')

        if type(scale_factor) is not float and type(scale_factor) is not int:
            raise ValueError("scale_factor must be a float or int")

        if type(min_neighbors) is not int:
            raise ValueError("min_neighbors must be a integer")

        if len(min_size) > 2:
            raise ValueError("min_size must be a tuple of two integers values")

        if any([True if type(i) is not int else False for i in min_size]):
            raise ValueError("min_size must be a tuple of two integers values")

        self.scale_factor = scale_factor
        self.min_neighbors = min_neighbors
        self.min_size = min_size
        self.face_cascade = cv2.CascadeClassifier(weights_file)

    def detect_faces(self, image):

        if image is None or not hasattr(image, "shape"):
            raise ValueError("Image not valid.")

        faces = self.face_cascade.detectMultiScale(
            image=cv2.cvtColor(image, cv2.COLOR_BGR2GRAY),
            scaleFactor=self.scale_factor,
            minNeighbors=self.min_neighbors,
            minSize=self.min_size)

        boxes = list()
        for face in faces:
            [x, y, w, h] = face.tolist()
            boxes.append([x, y, x + w, y + h])

        return [{'label': 'face', 'box': box} for box in boxes]


class EyeCascade(object):

    def __init__(self,
                 weights_file: str = None,
                 scale_factor: float = 1.1,
                 min_neighbors: int = 3,
                 min_size: tuple = (0, 0)):

        if weights_file is None:
            weights_file = resource_filename(
                Requirement.parse('cv2utils'),
                'cv2utils' + os.path.sep + 'data' + os.path.sep + 'haarcascade_eye.xml')

        if type(scale_factor) is not float and type(scale_factor) is not int:
            raise ValueError("scale_factor must be a float or int")

        if type(min_neighbors) is not int:
            raise ValueError("min_neighbors must be a integer")

        if len(min_size) > 2:
            raise ValueError("min_size must be a tuple of two integers values")

        if any([True if type(i) is not int else False for i in min_size]):
            raise ValueError("min_size must be a tuple of two integers values")

        self.scale_factor = scale_factor
        self.min_neighbors = min_neighbors
        self.min_size = min_size
        self.eye_cascade = cv2.CascadeClassifier(weights_file)

    def detect_eyes(self, image):

        if image is None or not hasattr(image, "shape"):
            raise ValueError("Image not valid.")

        eyes = self.eye_cascade.detectMultiScale(
            image=cv2.cvtColor(image, cv2.COLOR_BGR2GRAY),
            scaleFactor=self.scale_factor,
            minNeighbors=self.min_neighbors,
            minSize=self.min_size)

        boxes = list()
        for eye in eyes:
            [x, y, w, h] = eye.tolist()
            boxes.append([x, y, x + w, y + h])

        return [{'label': 'eye', 'box': box} for box in boxes]



from os import path
from setuptools import setup


def readme():
    directory = path.abspath(path.dirname(__file__))
    with open(path.join(directory, 'README.md'), encoding="UTF-8") as f:
        return f.read()


__version__ = "0.1.0"


setup(
    name='cv2utils',
    version=__version__,
    description='Implementation of some object detection',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/luizcarloscf/cv2utils.git',
    author="Luiz Carlos Cosmi Filho",
    author_email="luizcarloscosmifilho@gmail.com",
    license='MIT',
    packages=['cv2utils'],
    zip_safe=False,
    install_requires=[
        'opencv-contrib-python==4.2.0.32',
        'numpy==1.18.1',
    ],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    keywords="dnn haar cascade face eye detection opencv numpy pip package",
    include_package_data=True,
)
